@@ -6,6 +6,7 @@ while (<>) {
     my $f = $_;
     my @path = split(/\//, $f);
     my $lf = pop @path;
+    next unless $lf;
     if ($lf =~ /(.*)\.(\S+)$/) {
         my $n = $1;
         my $sfx = $2;
@@ -16,10 +17,19 @@ while (<>) {
               join('::',@path,$n);
             `mkdir -p $dir` unless -d $dir;
             my $outf = $dir . '/'. $n . '.html';
-            system("pod2html --htmlroot /dev/pod --title $title $f > $outf");
+            my $tmpf = "tmp";
+            open(OF,">$tmpf") || die ("can't open $tmpf");
+            open(F, $f) || die $f;
+            while(<F>) {
+                s/L\<(map2slim|go2\w+|\w+)(\.pl|)\>/L\<scripts::$1\>/g;
+                print OF $_;
+            }
+            close(F);
+            close(OF);
+            system("pod2html --htmlroot /dev/pod --title $title $tmpf > $outf");
         }
     }
-    elsif ($path[-1] eq 'scripts') {
+    elsif (@path && $path[-1] eq 'scripts') {
         print STDERR "Making pod for $lf\n";
         my $dir = join('/', 'pod', @path);
         my $title = 
